@@ -81,6 +81,16 @@ def is_wsl():
     return False
 
 
+def is_sailfish_os():
+    """ Return whether we are running on Sailfish OS """
+    if "linux" in platform.system().lower() and os.access("/etc/sailfish-release", os.R_OK):
+        with open("/etc/sailfish-release", "r") as f:
+            # Find 'ID=sailfishos'
+            if "sailfishos" in f.read():
+                return True
+    return False 
+
+
 def is_termux():
     """ Return whether we are running under the Termux application for Android"""
     return "com.termux" in os.environ["PATH"] and find_executable("termux-open-url")
@@ -129,6 +139,14 @@ def escape_fish_cmd(text):
     escaped = text.replace("\\", "\\\\").replace("'", "\\'")
     return "'" + escaped + "'"
 
+def strip_one_layer(text, char):
+    # Strip the text from one layer of a given character
+    if text[-1] == char:
+        text = text[:-1]
+    if text[0] == char:
+        text = text[1:]
+    return text
+
 
 named_colors = {
     "black": "000000",
@@ -1319,8 +1337,8 @@ class FishConfigHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             # Remove one layer of single-quotes because escape_fish_cmd adds them back.
             "abbr --add %s %s"
             % (
-                escape_fish_cmd(abbreviation["word"].strip("'")),
-                escape_fish_cmd(abbreviation["phrase"].strip("'")),
+                escape_fish_cmd(strip_one_layer(abbreviation["word"], "'")),
+                escape_fish_cmd(strip_one_layer(abbreviation["phrase"], "'")),
             )
         )
         if err:
@@ -1457,13 +1475,62 @@ class FishConfigHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
 
         if p == "/set_color/":
             print("# Colorscheme: " + postvars.get("theme"))
+            have_colors = set()
+            known_colors = set(
+                (
+                    "fish_color_normal",
+                    "fish_color_command",
+                    "fish_color_keyword",
+                    "fish_color_quote",
+                    "fish_color_redirection",
+                    "fish_color_end",
+                    "fish_color_error",
+                    "fish_color_param",
+                    "fish_color_option",
+                    "fish_color_comment",
+                    "fish_color_selection",
+                    "fish_color_operator",
+                    "fish_color_escape",
+                    "fish_color_autosuggestion",
+                    "fish_color_cwd",
+                    "fish_color_user",
+                    "fish_color_host",
+                    "fish_color_host_remote",
+                    "fish_color_cancel",
+                    "fish_color_search_match",
+                    "fish_pager_color_progress",
+                    "fish_pager_color_background",
+                    "fish_pager_color_prefix",
+                    "fish_pager_color_completion",
+                    "fish_pager_color_description",
+                    "fish_pager_color_selected_background",
+                    "fish_pager_color_selected_prefix",
+                    "fish_pager_color_selected_completion",
+                    "fish_pager_color_selected_description",
+                    "fish_pager_color_secondary_background",
+                    "fish_pager_color_secondary_prefix",
+                    "fish_pager_color_secondary_completion",
+                    "fish_pager_color_secondary_description",
+                )
+            )
             for item in postvars.get("colors"):
                 what = item.get("what")
                 color = item.get("color")
 
                 if what:
+                    if not what.startswith("fish_pager_color_") and not what.startswith(
+                        "fish_color_"
+                    ):
+                        have_colors.add("fish_color_" + what)
+                    else:
+                        have_colors.add(what)
                     output = self.do_set_color_for_variable(what, color)
 
+            # Set all known colors that weren't defined in this theme
+            # to empty, to avoid keeping around coloration from an earlier theme.
+            for what in known_colors - have_colors:
+                output += "\n" + self.do_set_color_for_variable(what, "")
+
         elif p == "/get_function/":
             what = postvars.get("what")
             output = [self.do_get_function(what[0])]
@@ -1622,6 +1689,10 @@ print(
     "Web config started at %s%s%s"
     % (esc["underline"], fileurl, esc["exit_attribute_mode"])
 )
+print(
+    "If that doesn't work, try opening %s%s%s"
+    % (esc["underline"], url, esc["exit_attribute_mode"])
+)
 print("%sHit ENTER to stop.%s" % (esc["bold"], esc["exit_attribute_mode"]))
 
 
@@ -1639,6 +1710,8 @@ def runThing():
         subprocess.call(["termux-open-url", url])
     elif is_chromeos_garcon():
         webbrowser.open(url)
+    elif is_sailfish_os():
+        subprocess.call(["xdg-open", url])
     else:
         webbrowser.open(fileurl)
 
